/*******************************************************************************************************************
Adopt, Discover and Improve

Dmitri V. Korotkevitch 
Sql Saturday #74 
(Jacksonville, FL. April 30, 2011)

Paging example
*******************************************************************************************************************/


declare
	@CustomerId int = 50
	
;with CTE
as
(
	select o.OrderId, o.OrderDate, o.OrderNum, o.Reference, os.Name as Status
		,pa.Address as PickupAddress, pa.City as PickupCity, pa.State as PickupState, pa.Zip as PickupZip
		,da.Address as DropoffAddress, da.City as DropoffCity, da.State as DropoffState, da.Zip as DropoffZip
		,s.Name as Service, r.Name as RatePlan, d.Name as Driver
		,o.Amount
		,ROW_NUMBER() over (order by o.OrderDate ASC, o.OrderStatusId asc) as RowNum
	from 
		Delivery.Orders o join Delivery.OrderStatuses os on 
			o.OrderStatusId = os.OrderStatusId
		join Delivery.Addresses pa on
			o.PickupAddressId = pa.AddressID
		join Delivery.Addresses da on
			o.DeliveryAddressId = da.AddressID
		join Delivery.[Services] s on
			o.ServiceId = s.ServiceId
		join Delivery.RatePlans r on
			o.RatePlanId = r.RatePlanId
		left join Delivery.Drivers d on
			o.DriverId = d.DriverId
	where o.CustomerId = 50 and o.OrderDate >= '2011-01-01' and o.Amount > 10
)
select * from CTE
where RowNum between 500 and 520
order by RowNum
go



;with Filter(OrderId, RowNum)
as
(
	select o.OrderId, ROW_NUMBER() over (order by o.OrderDate ASC, o.OrderStatusId asc) as RowNum
	from 
		Delivery.Orders o 
	where o.CustomerId = 50 and o.OrderDate >= '2011-01-01' and o.Amount > 10
)
,PagedData(OrderId, RowNum)
as
(
	select * 
	from Filter
	where RowNum between 500 and 520
)
select o.OrderId, o.OrderDate, o.OrderNum, o.Reference, os.Name as Status
	,pa.Address as PickupAddress, pa.City as PickupCity, pa.State as PickupState, pa.Zip as PickupZip
	,da.Address as DropoffAddress, da.City as DropoffCity, da.State as DropoffState, da.Zip as DropoffZip
	,s.Name as Service, r.Name as RatePlan, d.Name as Driver
	,o.Amount
	,ROW_NUMBER() over (order by o.OrderDate ASC, o.OrderStatusId asc) as RowNum
from 
	Delivery.Orders o join PagedData pd on
		o.OrderId = pd.OrderId
	join Delivery.OrderStatuses os on 
		o.OrderStatusId = os.OrderStatusId
	join Delivery.Addresses pa on
		o.PickupAddressId = pa.AddressID
	join Delivery.Addresses da on
		o.DeliveryAddressId = da.AddressID
	join Delivery.[Services] s on
		o.ServiceId = s.ServiceId
	join Delivery.RatePlans r on
		o.RatePlanId = r.RatePlanId
	left join Delivery.Drivers d on
		o.DriverId = d.DriverId
order by
	pd.RowNum
go

